C>>
C>> This program is to generate the POWER distribution file
C   for running DANTE induction heating models
C>>------------------------------------------------------------------
         PROGRAM IH_POWER_CREATE
C
       IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)

       CHARACTER FILNAM1*70, OFILE*70
       CHARACTER*255 TEXTA
       CHARACTER INSTNAM*70
C
       CHARACTER STRNG1*225, STRNG2*225
C
       DIMENSION FLUXA(2), TIMEA(2), COORDSA(3)
       CHARACTER*80 SNAME

C>> Read in the Carbon file from ABAQUS output Report 
       WRITE(*,*) 'Input Element-Integration-Coordinate File Name:'
       WRITE(*,*)
       READ(*,1000) FILNAM1 
C
C>> Output
C>> Open the iutput file name:
       NLEN=LEN_TRIM(FILNAM1)
       NFILE=0
       DO I=1,NLEN
         IF (FILNAM1(I:I).NE.'.' .AND. FILNAM1(I:I).NE.' ') THEN 
           NFILE=NFILE+1
         ELSE
           GOTO 120
         ENDIF
       END DO
120    CONTINUE
C
       OFILE=FILNAM1(1:NFILE)//'_POWER.OUT'
C> Open the output file for writing
       OPEN(16,FILE=OFILE, STATUS='UNKNOWN')
       WRITE(16,1000) '** Power File'
       WRITE(16,1010)  FILNAM1(1:NLEN)
       WRITE(16,1000) '** '

C>> Open the Element-coordinate file
       OPEN(UNIT=12, FILE=FILNAM1, STATUS='OLD', ERR=160)
       GOTO 180
160    CONTINUE
       WRITE(*,*) 'Can not open carbon input file, program terminated!'
       WRITE(*,*)  FILNAM1(1:NLEN)
       STOP
180    CONTINUE
C
C>>--------------------------------------------------------
C>> Processing the Input File
200    CONTINUE
       READ(12, 1000, ERR=3333) TEXTA
       GOTO 3334
3333   WRITE(7,*) 'Error after reading line: ', TEXTA
       WRITE(7,*) 'The line will be ignored!'
C>>       STOP
       GOTO 3500
3334   CONTINUE
C>> Change everything to upper case
       CALL TO_UCASE2(TEXTA)
       CALL ANAVAL(TEXTA, X0,Y0,Z0, IDATA,ID_ELE,ID_ITG)

      IF(IDATA.EQ.0) THEN  !not a data line, do nothing
C>>      
         NLEN1=LEN_TRIM(TEXTA)
         STRNG1='** '//TEXTA(1:NLEN1)
         WRITE(16,1000) STRNG1
C
       ELSE IF(IDATA.EQ.1) THEN  !is data line
C
         FLUXA(1)=0.D0
         FLUXA(2)=0.D0
C
         TIMEA(1)=0.D0
         TIMEA(2)=0.D0
C
         COORDSA(1)=X0
         COORDSA(2)=Y0
         COORDSA(3)=Z0
C
         KSTEP=1
         KINC=1
         NOEL=ID_ELE
         NPT=ID_ITG
         
         CALL DFLUX(FLUXA,SOL,KSTEP,KINC,TIMEA,NOEL,NPT,
     &     COORDSA,JLTYP,TEMP,PRESS,SNAME)


         WRITE(STRNG1, 1500) ID_ELE, X0, Y0, Z0, FLUXA(1)
1500     FORMAT(I12,',',2X, 4(E19.6,','))
C
C>> String Process
         CALL STRNG_PROCESS(STRNG1,STRNG2,NSTRNG)
         WRITE(16,1000) STRNG2(1:NSTRNG)
C
       ELSE
         WRITE(7,*) 'Unexpected Error, Stop!'
         STOP
       ENDIF
C>> Read till the end of the file
3500   CONTINUE
       IF(.NOT. EOF(12)) GOTO 200


C>> Close the input file
       CLOSE(12)


C----checking was done
95     CONTINUE  
C>> CLOSE Input File
       CLOSE(12)

C>> CLOSE Output File
C       WRITE(16,1000) '** End Of File'
       CLOSE(16)
       WRITE(*,*) 'Program done'


       STOP
C>>-----------------------------------------------------------------
1000   FORMAT(A)
1010   FORMAT('** Input Carbon File name:  ', A)
C>>
       END

C>>------------------------------------------------------------------
C>> Dlfux single shot Abaqus IH Example Model, R=12.7mm
       SUBROUTINE DFLUX(FLUX,SOL,KSTEP,KINC,TIME,NOEL,NPT,
     &     COORDS,JLTYP,TEMP,PRESS,SNAME)

       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)
C
       DIMENSION FLUX(2), TIME(2),COORDS(3)
       CHARACTER*80 SNAME
C>>-----------------------------
       DIMENSION HGEN(10)
       DIMENSION HDEPTH(10)
C>>-----------------------------
       DIMENSION HGENA(10)
       DIMENSION HDEPTHA(10)
C>> For  Recipe A
       DATA (HDEPTHA(I),I=1,10) /0.25D0, 0.5D0, 0.75D0, 1.D0, 5.D0,
     &                          6.D0, 7.D0, 8.D0, 9.D0, 10.D0 /
       DATA (HGENA(I),I=1,10) /5.250D0,2.75D0,1.25D0,0.15D0,0.0D0,
     &                        0.00D0,0.00D0,0.00D0,0.00D0,0.00D0 /
C
C Recipe J
       DIMENSION HGENJ(10)
       DIMENSION HDEPTHJ(10)
       DATA (HDEPTHJ(I),I=1,10) /0.D0, 1.D0, 2.D0, 3.D0, 4.D0,
     &                          5.D0, 6.D0, 7.D0, 8.D0, 9.D0 /
C>> For Journal-2 Recipe J
       DATA (HGENJ(I),I=1,10) /5.80D0,1.00D0,0.00D0,0.00D0,0.0D0,
     &                        0.00D0,0.00D0,0.00D0,0.00D0,0.00D0 /
C>>-----------------------------
C>>--------------------------------------------------------------------
       FLUX(1) = 0.D0
       FLUX(2) = 0.D0
C
C>> The induction heating is only used in the first step for this set up.
C>> Preheat
         WEIGHT=1.0D0  !Recipe A
         DO I=1, 10
           HGEN(I) = HGENA(I)
           HDEPTH(I) = HDEPTHA(I)
         END DO
C
C>> Find the heat flux for the specific integration point
       X0=COORDS(1)
       Y0=COORDS(2)
       Z0=COORDS(3)
C
C>>-------------------------------------------------------------------
C>> Calculate the flux energy base on the depth of the point
C>> In the axial direction, no power outside of [-76, 76] 
       IF((Y0.LT.-100.D0) .OR. (Y0.GT.100.D0)) RETURN   !outside Induction
C
C>> Power linear interpolation from the mid to counter weight 
C       DIST=DABS(Y0-0.D0)
C
       WEIGHT= (DIST/60.0D0)*0.5+1.D0
C
C>>-------------------------------------------------------------------

C>> Calculate the flux energy base on the depth of the point
C>> Cranskahy model
       RR=DSQRT(X0**2+Z0**2)
       DEPTH=60.0D0 - RR

C       IF(((Y0.GT.101.D0) .AND. (Z0.LE.0.D0)) .OR.   !outside Induction
C    &   ((Z0.GE.0.D0) .AND. (Z0.LT.1.D0))) THEN
C
C         IF(DEPTH.LT.0.D0 .AND. DEPTH.GT.-20.D0) THEN
C           WEIGHT=1.5D0
C           DEPTH=DABS(Z0)-12.7D0
C           IF (DEPTH .LT. 0.D0) DEPTH = 0.D0
C         ELSE IF(DEPTH.LT.-20.D0) THEN
C           WEIGHT=0.D0
C         ENDIF  
       
C      ENDIF
C
C>>
       IF(DEPTH.GT.10.D0) RETURN
C
C>> Points for interpolation
       JP1=-1
C       IF(DEPTH.GE.0.D0 .AND. DEPTH.LT.HDEPTH(1)) THEN
       IF(DEPTH.LT.HDEPTH(1)) THEN
         JP1 = 1
       ELSE IF(DEPTH.GE.HDEPTH(1) .AND. DEPTH.LT.HDEPTH(2)) THEN
         JP1 = 2
       ELSE IF(DEPTH.GE.HDEPTH(2) .AND. DEPTH.LT.HDEPTH(3)) THEN
         JP1 = 3
       ELSE IF(DEPTH.GE.HDEPTH(3) .AND. DEPTH.LT.HDEPTH(4)) THEN
         JP1 = 4
       ELSE IF(DEPTH.GE.HDEPTH(4) .AND. DEPTH.LT.HDEPTH(5)) THEN
         JP1 = 5
       ELSE IF(DEPTH.GE.HDEPTH(5) .AND. DEPTH.LT.HDEPTH(6)) THEN
         JP1 = 6
       ELSE IF(DEPTH.GE.HDEPTH(6) .AND. DEPTH.LT.HDEPTH(7)) THEN
         JP1 = 7
       ELSE IF(DEPTH.GE.HDEPTH(7) .AND. DEPTH.LT.HDEPTH(8)) THEN
         JP1 = 8
       ELSE IF(DEPTH.GE.HDEPTH(8) .AND. DEPTH.LT.HDEPTH(9)) THEN
         JP1 = 9
       ELSE IF(DEPTH.GE.HDEPTH(9) .AND. DEPTH.LE.HDEPTH(10)) THEN
         JP1 = 10
       ENDIF
C
C>> Interpolation
       IF(JP1.LE.1) THEN
         FLUX(1) = HGEN(1)
       ELSE IF(JP1.GT.10) THEN
         FLUX(1) = 0.D0
       ELSE !JP1 >=2, <=10, Interpolation
         FLUX(1)=HGEN(JP1-1)+(HGEN(JP1)-HGEN(JP1-1))*
     &           (DEPTH-HDEPTH(JP1-1))/(HDEPTH(JP1)-HDEPTH(JP1-1))
       ENDIF
C>> Adding Weight
       FLUX(1)=FLUX(1)*WEIGHT
C
C>>
       RETURN
       END
C
C>>********************************************************************
         SUBROUTINE TO_UCASE2(WORD)
       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)
C>> Convert lower case string to upper case string
       CHARACTER*255 WORD

       NLEN = LEN_TRIM(WORD)
       DO I=1,NLEN
         IC = ICHAR(WORD(I:I))
         IF(IC.GE.97 .AND. IC.LE.122) WORD(I:I)=CHAR(IC-32)
       END DO

       RETURN
       END

C>>********************************************************************
         SUBROUTINE ANAVAL(TXTS,X0,Y0,Z0, IDATA,ID_ELE,ID_ITG)
       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)

       CHARACTER*255 TXTS
C>>------------------------------------------------------------------
       IDATA=0  !not a data line

       NLEN1=LEN_TRIM(TXTS)
       IF(NLEN1.EQ.0) RETURN
       
C      READ(TXTS, *,ERR=3333) ID_ELE, ID_ITG, X0,Y0,Z0
       READ(TXTS, *,ERR=3333) ID_ELE, X0,Y0,Z0
       GOTO 3334
3333   CONTINUE
       ID_ELE = 0
       ID_ITG = 0
       X0=0.D0
       Y0=0.D0
       Z0=0.D0
C
       RETURN
       
3334   CONTINUE
       IF(ID_ELE.GT.0) IDATA=1  !is data line

       RETURN
       END
C
C>>********************************************************************
       SUBROUTINE STRNG_PROCESS(STRG1, STRG2, NLENG)
       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)

       CHARACTER STRG1*225, STRG2*225

       LTOT=LEN_TRIM(STRG1)

       NLENG=0 
       DO I=1, LTOT
         IF(STRG1(I:I) .NE. ' ') THEN
           IF(STRG1(I:I) .EQ. ',') THEN
             NLENG=NLENG+1 
             STRG2(NLENG:NLENG)=STRG1(I:I)
C
             STRG2(NLENG+1:NLENG+4)='    '
             NLENG=NLENG+4 
           ELSE
             NLENG=NLENG+1 
             STRG2(NLENG:NLENG)=STRG1(I:I)
           ENDIF
         ENDIF
       END DO

       RETURN
       END
C>>********************************************************************

